from bson import ObjectId
from cerberus import Validator
import datetime
from dgapi import DGAPI
from flask import jsonify, make_response, request, session
from functools import wraps
import jwt
from logging import error
from mongoCon import MongoCon
from utils.helpers import CustomErrorHandler, remove_dots


def is_authorize():
    return DGAPI.get('valid_token').status_code == 200


def login_required(f):
    """Full login"""

    @wraps(f)
    def decorated(*args, **kwargs):
        token = request.headers.get('security-token')
        try:
            data = jwt.decode(token, '@3!f3719em$893&')  # token decode
        except Exception:   # jwt.ExpiredSignatureError
            error('TOKEN NO VALID')
            return make_response(jsonify({'error': 'Unauthorized access'}), 401)
        else:  # token valido
            session['user_id'] = data['user_id']
            session['user_application_id'] = data['user_application_id']
            session['user_name'] = data['name']
            session['user_email'] = data.get('email', '')
            session['roles'] = data.get('security_actions', [])
            if is_authorize():
                return f(*args, **kwargs)
            else:
                return make_response(jsonify({'error': 'Unauthorized access'}), 401)
    return decorated


def login_script(f):

    @wraps(f)
    def decorated(*args, **kwargs):
        token = request.headers.get('token', '')
        session['token'] = token
        if token == '5gABTw3tXCS{]NN[fk,dUWKcmr3,DaHoa[hiSTfVzq.cZp;W72B[CU.7,kqA(}':
            return f(*args, **kwargs)
        else:
            return make_response(jsonify({'error': 'Unauthorized access'}), 401)
    return decorated


def activity_log_decorator(msg=None):
    """
    Decorador para guardar en el log una entrada a un endpoint
    msg: Es un mensaje opcional
    """
    def inner_function(f):
        @wraps(f)
        def wrapper(*args, **kwargs):
            data_log = {
                'user': session.get('user_name', 'Anonymous'),
                'endpoint': request.path,
                'values': remove_dots(request.json),
                'date': datetime.datetime.now(),
                'server_protocol': request.environ.get('SERVER_PROTOCOL', None),
                'request_method': request.environ.get('REQUEST_METHOD', None),
                'remote_addr': request.environ.get('REMOTE_ADDR', None),
                'remote_port': request.environ.get('REMOTE_PORT', None),
                'content_type': request.environ.get('CONTENT_TYPE', None),
                'http_user_agent': request.environ.get('HTTP_USER_AGENT', '')
            }
            if msg is not None:
                data_log['message'] = msg
            with MongoCon() as cnx:
                cnx.activity_log.insert_one(data_log)
            return f(*args, **kwargs)
        return wrapper
    return inner_function


def validate_request(schema=None, origin='json', headers=False):
    """
    Decorador para validar un el request
    - schema: diccionario con el schema que debe cumplir la petición
    Por default los campos son requeridos
    """
    def inner_function(f):
        @wraps(f)
        def wrapper(*args, **kwargs):
            response = {}
            if headers:
                if isinstance(headers, bool):
                    local_headers = ["current-group", "current-dealer"]
                else:
                    local_headers = headers
                headers_missing = [header for header in local_headers if header not in request.headers]
                if len(headers_missing) > 0:
                    response['headers'] = {}
                    for param in headers_missing:
                        response['headers'][param] = 'Required header not found'
            if schema:
                schema_r = schema
                if origin == 'form':
                    validate_data = request.form.copy() if request.form else {}
                else:
                    validate_data = request.json.copy() if request.json else {}
                if isinstance(schema_r, dict):
                    if 'type' in schema and 'schema' in schema:
                        schema_r = {'data': schema}
                        validate_data = {'data': validate_data}
                    for value in schema_r.values():
                        if isinstance(value, dict):
                            if 'required' not in value:
                                value.update({'required': True})
                            if 'type' in value:
                                if value['type'] == 'custom_date':
                                    del value['type']
                                    format_ = value.pop('format', '%Y-%m-%d')
                                    value.update({
                                        'type': 'date',
                                        'coerce': lambda s: datetime.datetime.strptime(s, format_)
                                    })
                                elif value['type'] == 'object_id':
                                    del value['type']
                                    value.update({
                                        'type': 'string',
                                        'coerce': lambda s: str(ObjectId(s) if s else '')
                                    })
                else:
                    raise TypeError('The schema should be a dict')
                v = Validator(schema_r, error_handler=CustomErrorHandler)
                v.allow_unknown = True
                if not v.validate(validate_data) or response:
                    response = make_response(jsonify({**response, **v.errors}), 422)
            if response == {}:
                response = f(*args, **kwargs)
            return response
        return wrapper
    return inner_function


def validate_args(params):
    """
    Decorador para validar los parametros de un endpoint
    - params: array de strings con los parametros que debe contener la peticion
    """
    def inner_function(f):
        @wraps(f)
        def wrapper(*args, **kwargs):
            v = Validator()
            response = {}
            if v.validate({'data': params}, {'data': {'type': 'list', 'schema': {'type': 'string'}}}):
                params_missing = [param for param in params if param not in request.args]
                if len(params_missing) > 0:
                    response['params'] = {}
                    for param in params_missing:
                        response['params'][param] = 'Required field not found'
                    response = make_response(jsonify(response), 422)
                else:
                    response = f(*args, **kwargs)
            else:
                raise TypeError('The params should be a list of strings')
            return response
        return wrapper
    return inner_function



from flask import jsonify, make_response


def success_ok():
    """Retorna un mensaje de exito"""
    return make_response(jsonify({"message": "success"}), 200)


def not_found(item=''):
    """Retorna un mensaje de no encontrado"""
    return make_response(jsonify({"message": "{} not found".format(item)}), 404)


def error_message(message):
    """Retorna un mensaje de error"""
    return make_response(jsonify({"message": message}), 406)


def error_params_required(params=[]):
    """
    Recibe un array con los parametros faltantes
    Retorna un mensaje con los parametros faltantes
    """
    message = {}
    for param in params:
        message[param] = 'Required field not found'
    return make_response(jsonify({"message": message}), 422)



# manejo de cola (NO ORDENAR)
import eventlet
eventlet.monkey_patch()

# dependencias
from flask import Flask, jsonify
from flask_socketio import SocketIO
from const import ENV, MONGO_CONN
from utils.helpers import CustomJSONEncoder
import json


# instancia de aplicacion con socket
app = Flask(__name__)
app.secret_key = "@3!f3719em$893&"
app.json_encoder = CustomJSONEncoder
json.JSONEncoder = CustomJSONEncoder
socketio = SocketIO(app, cors_allowed_origins="*", message_queue='redis://', async_mode='eventlet')


# saludo
@app.route('/', methods=['GET'])
def index():
    return jsonify({"message": "Welcome to <<API_NAME>> API", "ENV": ENV, "db": MONGO_CONN['database']})



from flask import request, session
from bson import ObjectId
from cerberus import errors
from datetime import date, datetime, timedelta
import decimal
import json
from os import path
from random import choice
from string import digits, ascii_letters
from traceback import StackSummary, walk_tb
from const import ROOT_API
from mongoCon import MongoCon


class CustomJSONEncoder(json.JSONEncoder):

    DATE_FORMAT = r'%Y-%m-%d'
    DATETIME_FORMAT = r'%Y-%m-%d %H:%M:%S'

    def default(self, obj):
        if isinstance(obj, decimal.Decimal):
            return round(float(obj), 2)
        elif isinstance(obj, datetime):
            return obj.strftime(self.DATETIME_FORMAT)
        elif isinstance(obj, date):
            return obj.strftime(self.DATE_FORMAT)
        elif isinstance(obj, ObjectId):
            return str(obj)
        return super().default(obj)


class CustomErrorHandler(errors.BasicErrorHandler):
    messages = errors.BasicErrorHandler.messages.copy()
    messages[errors.REQUIRED_FIELD.code] = 'required field not found'


def create_string(ln=15, allow_special=False):
    """Create random string"""
    special_char = "!#$%&*+-=?@^_|"
    if allow_special:
        string = ''.join(choice(ascii_letters + digits + special_char) for _ in range(ln))
    else:
        string = ''.join(choice(ascii_letters + digits) for _ in range(ln))
    return string


def last_day_of_month(month, year):
    """
    Esta funcion retorna el ultimo día del mes dado
    - month: numero de mes
    - year: numero del
    """
    new_month = month + 1 if month < 12 else 1
    new_year = year if month < 12 else year + 1
    return datetime(new_year, new_month, 1) - timedelta(seconds=1)


def date_range_query(start_date, end_date, date_format: str = '%Y-%m-%d'):
    """
    This function return a dictionary with the format to search in the range of start_date and end_date
    - start_date(string): Fecha inicial del rango a buscar
    - end_date(string): Fecha final del rango a buscar
    - format(string): Formato en el que se encuentran las fechas
    """
    return {
        '$gte': datetime.strptime(start_date, date_format),
        '$lt': datetime.strptime(end_date, date_format) + timedelta(days=1) - timedelta(milliseconds=1)
    }


def month_range_query(month, year):
    """
    Esta funcion retorna un diccionario con el formato para hacer una busqueda por todas las fechas del mes dado
    - month: mes del año
    - year: year a buscar
    """
    return {
        '$gte': datetime(year, month, 1),
        '$lt': last_day_of_month(month, year)
    }


def find_in_arr_by_dict(arr, search_dict, key_name='_id'):
    """
    Esta funcion busca un elemento en el array apartir de un diccionario de busqueda, sino lo encuentra retorna None
    - arr: array de diccionarios
    - search_dict: diccionario de datos a buscar
    - key_name(opcional): Llave sobre la que se hara la busqueda. La llave sobre la que se busca tambien debe ser un diccionario
    """
    result = None
    for element in arr:
        add = True
        for key in search_dict:
            if key not in element[key_name] or element[key_name][key] != search_dict[key]:
                add = False
        if add:
            result = element
            break
    return result


def docs_path(controller, filename):
    """
    Esta funcion regresa el directorio donde se guardaran las plantillas para la documentación
    de un controlador
    - controller: nombre del controlador
    - filename: nombre del archivo
    """
    return path.join(ROOT_API, "modules", controller, "docs", filename)


def valid_filename_or_next(directory, file_name):
    """
    Esta funcion valida que el file_name no exista en el directorio, si existe, busca el siguiente nombre disponible
    - directory: Directorio donde se validara el archivo
    - file_name: Nombre del archivo a validar
    """
    new_file_name = file_name
    if path.exists(path.join(directory, file_name)):
        version = 1
        valid = False
        file, ext = file_name.rsplit('.', 1)
        while not valid:
            new_file_name = "{} ({}).{}".format(file, version, ext)
            if path.exists(path.join(directory, new_file_name)):
                version += 1
            else:
                valid = True
    return new_file_name


def remove_dots(data):
    """
    Esta funcion remueve los puntos
    - data: diccionario
    """
    if isinstance(data, dict):
        return {remove_dots(key): value if isinstance(value, str) else remove_dots(value) for key, value in data.items()}
    elif isinstance(data, list):
        return [remove_dots(element) for element in data]
    elif isinstance(data, str):
        return data.replace('.', '||')
    else:
        return data


def error_manager(error):
    """
    Esta funcion retorna un diccionario con los datos del error
    - error
    """
    stack_trace = []
    for trace in StackSummary.extract(walk_tb(error.__traceback__), capture_locals=True):
        stack_trace.append({
            'File': trace.filename,
            'Line': trace.lineno,
            'Function': trace.name,
            'Message': trace.line,
            'Locals': trace.locals
        })
    user = session.get('user_name', request.headers.get('user-logged-in', 'Anonymous'))
    return {
        'endpoint': request.path,
        'values': remove_dots(request.json),
        'Error type': type(error).__name__,
        'Error message': str(error),
        'Stack trace': stack_trace,
        'user': user,
        'date': datetime.now(),
    }


def activity_log_function(collection, action, data, msg=None):
    """
    Funcion para guardar en el log una acción que modifique la base de datos
    collection: Colleción que sera modificada
    action: Acción realizada sobre la colección
    data: Datos utilizados
    msg: Es un mensaje opcional
    """
    user = request.headers.get('user-logged-in', 'Anonymous')
    data_log = {
        'user': user,
        'endpoint': request.path,
        'collection': collection,
        'action': action,
        'data': remove_dots(data),
        'date': datetime.datetime.now(),
        'server_protocol': request.environ.get('SERVER_PROTOCOL', None),
        'request_method': request.environ.get('REQUEST_METHOD', None),
        'remote_addr': request.environ.get('REMOTE_ADDR', None),
        'remote_port': request.environ.get('REMOTE_PORT', None),
        'content_type': request.environ.get('CONTENT_TYPE', None),
        'http_user_agent': request.environ.get('HTTP_USER_AGENT', '')
    }
    if msg is not None:
        data_log['message'] = msg
    with MongoCon() as cnx:
        cnx.activity_log.insert_one(data_log)



from setuptools import setup, find_packages

long_desc = ""
with open('./README.md', encoding="utf-8") as fl:
    long_desc = fl.read()

setup(
    name='dgx',
    packages=find_packages(
        include=['dgx', 'dgx.*']
    ),
    version='0.0.11',
    license='MIT',
    description='Dealergeek API',
    long_description=long_desc,
    long_description_content_type='text/markdown',
    author='Jeff Aguilar',
    author_email='jeff.aguilar.06@gmail.com',
    url='https://github.com/jaguilar08/dgx',
    download_url='https://github.com/jaguilar08/dgx/archive/refs/tags/0.1.tar.gz',
    keywords=['DEALERGEEK', 'API'],
    install_requires=[
        'Werkzeug',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'dgx = dgx.__main__:main',
        ]
    }
)



from sys import argv
from os import path
from logging import basicConfig, warning

MODE_DEBUG = '--debug' in argv

basicConfig(level="WARNING")

# directorios
ROOT_API = path.dirname(path.realpath(__file__))
RESOURCES_FILE_PATH = path.join(ROOT_API, 'resources')
DIRECTORY_TEMP_PATH = path.join(ROOT_API, "__temp__")

# envirenment
ENV = "DEV"

# puerto
PORT = <<PORT>>

# identificador de la aplicacion
APPLICATION_CODE = '<<APPLICATION_CODE>>'

# IP Local servidor api dealergeek
DGAPI_SERVER = ''


# cadena de coneccion
MONGO_CONN = {
    'user': '',
    'password': '',
    'host': '',
    'database': '',
    'port': 27017
}

if '--prod' in argv:
    ENV = "PROD"
    API_SERVER = ''
    MONGO_CONN.update({
        'password': '',
        'database': ''
    })
elif '--demo' in argv:
    ENV = "DEMO"
    API_SERVER = ''
    MONGO_CONN.update({
        'password': '',
        'database': ''
    })
warning("%s MODE" % ENV)
